@@ -1,8 +1,11 @@
-# $Id: Mrp.pm,v 1.3 2006/05/19 02:08:59 rvosa Exp $
+# $Id: Mrp.pm 2108 2006-08-29 20:46:17Z rvosa $
 # Subversion: $Rev: 190 $
 package Bio::Phylo::Unparsers::Mrp;
 use strict;
-use base 'Bio::Phylo::IO';
+use Bio::Phylo::IO;
+
+use vars '@ISA';
+@ISA=qw(Bio::Phylo::IO);
 
 # One line so MakeMaker sees it.
 use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
@@ -156,7 +159,7 @@ and then you'll automatically be notified of progress on your bug as I make
 changes. Be sure to include the following in your request or comment, so that
 I know what version you're using:
 
-$Id: Mrp.pm,v 1.3 2006/05/19 02:08:59 rvosa Exp $
+$Id: Mrp.pm 2108 2006-08-29 20:46:17Z rvosa $
 
 =head1 AUTHOR
 
