@@ -1,10 +1,12 @@
-# $Id: Newick.pm,v 1.21 2006/05/19 02:08:59 rvosa Exp $
+# $Id: Newick.pm 3387 2007-03-25 16:06:50Z rvosa $
 # Subversion: $Rev: 190 $
 package Bio::Phylo::Unparsers::Newick;
 use strict;
-use warnings;
 use Bio::Phylo::Forest::Tree;
-use base 'Bio::Phylo::IO';
+use Bio::Phylo::IO;
+
+use vars '@ISA';
+@ISA=qw(Bio::Phylo::IO);
 
 # One line so MakeMaker sees it.
 use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
@@ -102,7 +104,7 @@ sub _to_string {
                 $string = $n->get_name . ':' . $n->get_branch_length . ';';
             }
             else {
-                $n->get_name ? $string = $n->get_name . ';' : $string = ';';
+                $string = $n->get_name ? $n->get_name . ';' : ';';
             }
         }
         elsif ( !$n->get_previous_sister ) {
@@ -162,7 +164,7 @@ and then you'll automatically be notified of progress on your bug as I make
 changes. Be sure to include the following in your request or comment, so that
 I know what version you're using:
 
-$Id: Newick.pm,v 1.21 2006/05/19 02:08:59 rvosa Exp $
+$Id: Newick.pm 3387 2007-03-25 16:06:50Z rvosa $
 
 =head1 AUTHOR
 
